////////////////////////////////////////////////////////
//
// GEM - Graphics Environment for Multimedia
//
// zmoelnig@iem.kug.ac.at
//
// Implementation file
//
//    Copyright (c) 2009 IOhannes m zmoelnig. forum::fr::umlute. IEM
//    For information on usage and redistribution, and for a DISCLAIMER OF ALL
//    WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.
//
/////////////////////////////////////////////////////////

///////////////
// TODO
//
// - let the user define ROI


#include "pix_opencv_contours.h"
#include <stdio.h>

CPPEXTERN_NEW(pix_opencv_contours)

/////////////////////////////////////////////////////////
//
// pix_opencv_contours
//
/////////////////////////////////////////////////////////
// Constructor
//
/////////////////////////////////////////////////////////
pix_opencv_contours :: pix_opencv_contours() :
	comp_xsize(0), comp_ysize(0),
	x_cmode(CV_RETR_LIST),
	x_cmethod(CV_CHAIN_APPROX_SIMPLE),
	m_approxAccuracy(3.),
	m_approx2(1.),
	orig(NULL), gray(NULL)
{ 
  m_dataout = outlet_new(this->x_obj, 0);
  m_countout = outlet_new(this->x_obj, 0);
}

/////////////////////////////////////////////////////////
// Destructor
//
/////////////////////////////////////////////////////////
pix_opencv_contours :: ~pix_opencv_contours()
{ 
	//Destroy cv_images to clean memory
	cvReleaseImage(&orig);
	cvReleaseImage(&gray);

  outlet_free(m_dataout);
	outlet_free(m_countout);
}

void pix_opencv_contours :: processCVImage()
{
  char tindex[4];
  int im = 0;                  // Indicator of markers.

	CvSeq* contours;
	CvMemStorage* stor02;
	stor02 = cvCreateMemStorage(0);

	cvFindContours( gray, stor02, &contours, sizeof(CvContour), x_cmode, x_cmethod, cvPoint(0,0) );
	if (contours) contours = cvApproxPoly( contours, sizeof(CvContour), stor02, CV_POLY_APPROX_DP, m_approxAccuracy, m_approx2 );


	CvSeq*dummy=contours;
	int numcontours=0;
	for( ; dummy != 0; dummy = dummy->h_next ) {
		numcontours++;
	}
	outlet_float( m_countout, numcontours );
	int contour_id=0;
	for( ; contours != 0; contours = contours->h_next )
    {
			int count = contours->total; // This is number point in contour
			t_atom ap[1+2*count];
			int i;
			SETFLOAT(ap, contour_id);
			for(i=0; i<count; i++) {
				CvPoint*point=(CvPoint*)cvGetSeqElem( contours, i );
				t_float x= (t_float)point->x;
				t_float y= (t_float)point->y;
				SETFLOAT(ap+2*i+1, x);
				SETFLOAT(ap+2*i+2, y);
			}			
			outlet_list( m_dataout, 0, 1+2*count, ap );
			contour_id++;
		}
	cvReleaseMemStorage( &stor02 );

}


/////////////////////////////////////////////////////////
// processImage
//
/////////////////////////////////////////////////////////
void pix_opencv_contours :: processRGBAImage(imageStruct &image)
{
  if ((comp_xsize!=image.xsize)||(comp_ysize!=image.ysize)||(!orig)) {

		comp_xsize = image.xsize;
		comp_ysize = image.ysize;

		//Destroy cv_images to clean memory
		cvReleaseImage(&orig);
		cvReleaseImage(&gray);

		//create the orig image with new size
		orig = cvCreateImage(cvSize(image.xsize,image.ysize), IPL_DEPTH_8U, 4);
		gray = cvCreateImage(cvSize(orig->width,orig->height), IPL_DEPTH_8U, 1);
	}

	// LATER use Gem's built'in functions for this
	memcpy( orig->imageData, image.data, image.xsize*image.ysize*4 );
    
	// Convert to grayscale
	cvCvtColor(orig, gray, CV_RGBA2GRAY);

	// do the processing 
	processCVImage();
}

void pix_opencv_contours :: processRGBImage(imageStruct &image)
{
  unsigned char *pixels = image.data;
  char tindex[4];
  int im = 0;                  // Indicator of markers.

  if ((comp_xsize!=image.xsize)||(comp_ysize!=image.ysize)||(!orig)) {

		comp_xsize = image.xsize;
		comp_ysize = image.ysize;

		//Destroy cv_images to clean memory
		cvReleaseImage(&orig);
		cvReleaseImage(&gray);

		//create the orig image with new size
		orig = cvCreateImage(cvSize(image.xsize,image.ysize), IPL_DEPTH_8U, 3);
		gray = cvCreateImage(cvSize(orig->width,orig->height), IPL_DEPTH_8U, 1);
	}


	// LATER use Gem's built'in functions for this
	memcpy( orig->imageData, image.data, image.xsize*image.ysize*4 );
    
	// Convert to grayscale
	cvCvtColor(orig, gray, CV_RGBA2GRAY);

	// do the processing 
	processCVImage();

}
    	
void pix_opencv_contours :: processGrayImage(imageStruct &image)
{ 
  char tindex[4];
  int im = 0;                  // Indicator of markers.

  if ((comp_xsize!=image.xsize)||(comp_ysize!=image.ysize)||(!orig)) {

		comp_xsize = image.xsize;
		comp_ysize = image.ysize;

		//Destroy cv_images to clean memory
		cvReleaseImage(&orig);
		cvReleaseImage(&gray);

		//create the orig image with new size
		orig = cvCreateImage(cvSize(image.xsize,image.ysize), IPL_DEPTH_8U, 1);
		gray =    cvCreateImage(cvSize(orig->width,orig->height), IPL_DEPTH_8U, 1);
	}

	//copy back the processed frame to image
	memcpy( gray->imageData, image.data, image.xsize*image.ysize );

	// do the processing 
	processCVImage();

}

void pix_opencv_contours :: floatCModeMess (float cmode)
{
	// CV_RETR_EXTERNAL || CV_RETR_LIST || CV_RETR_CCOMP || CV_RETR_TREE
	int mode = (int)cmode;

	switch(mode) {
	case ( CV_RETR_EXTERNAL ):
		x_cmode = CV_RETR_EXTERNAL;
		post( "mode set to CV_RETR_EXTERNAL" );
		break;
	case ( CV_RETR_LIST ):
		x_cmode = CV_RETR_LIST;
		post( "mode set to CV_RETR_LIST" );
		break;
	case ( CV_RETR_CCOMP ):
		x_cmode = CV_RETR_CCOMP;
		post( "mode set to CV_RETR_CCOMP" );
		break;
	case ( CV_RETR_TREE ):
		x_cmode = CV_RETR_TREE;
		post( "mode set to CV_RETR_TREE" );
		break;
	default:
		error( "ignoring unknown mode %d", mode);
		break;
	}
}

void pix_opencv_contours :: floatCMethodMess (float cmethod)
{
  int method = (int)cmethod;

	switch(method) {
	case ( CV_CHAIN_CODE ):
		error( "not supported method : CV_CHAIN_CODE" );
    break;
	case ( CV_CHAIN_APPROX_NONE ):
		x_cmethod = CV_CHAIN_APPROX_NONE;
		post( "method set to CV_CHAIN_APPROX_NONE" );
    break;
	case ( CV_CHAIN_APPROX_SIMPLE ):
		x_cmethod = CV_CHAIN_APPROX_SIMPLE;
		post( "method set to CV_CHAIN_APPROX_SIMPLE" );
    break;
	case ( CV_CHAIN_APPROX_TC89_L1 ):
		x_cmethod = CV_CHAIN_APPROX_TC89_L1;
		post( "method set to CV_CHAIN_APPROX_TC89_L1" );
    break;
	case ( CV_CHAIN_APPROX_TC89_KCOS ):
		x_cmethod = CV_CHAIN_APPROX_TC89_KCOS;
		post( "method set to CV_CHAIN_APPROX_TC89_KCOS" );
    break;
	case  ( CV_LINK_RUNS ):
		if(CV_RETR_LIST == x_cmode) {
			x_cmethod = CV_LINK_RUNS;
			post( "method set to CV_LINK_RUNS" );
		} else {
			post("ignoring CV_LINK_RUNS, since cmode must be %d", CV_RETR_LIST);
		}
    break;
	default:
		error("unknown method: %d", method);
		break;
	}
}

void pix_opencv_contours :: floatAccMess (float accuracy)
{
	m_approxAccuracy=accuracy;
}
void pix_opencv_contours :: floatApprox2Mess (float approx2)
{
	m_approx2=approx2;
}


/////////////////////////////////////////////////////////
// static member function
//
/////////////////////////////////////////////////////////
void pix_opencv_contours :: obj_setupCallback(t_class *classPtr)
{
  class_addmethod(classPtr, (t_method)&pix_opencv_contours::floatCModeMessCallback,
									gensym("mode"), A_FLOAT, A_NULL);
  class_addmethod(classPtr, (t_method)&pix_opencv_contours::floatCMethodMessCallback,
									gensym("method"), A_FLOAT, A_NULL);

  class_addmethod(classPtr, (t_method)&pix_opencv_contours::floatApprox1MessCallback,
									gensym("accuracy"), A_FLOAT, A_NULL);
  class_addmethod(classPtr, (t_method)&pix_opencv_contours::floatApprox2MessCallback,
									gensym("approx2"), A_FLOAT, A_NULL);

}

void pix_opencv_contours :: floatCModeMessCallback(void *data, t_floatarg cmode)
{
  GetMyClass(data)->floatCModeMess((float)cmode);
}

void pix_opencv_contours :: floatCMethodMessCallback(void *data, t_floatarg cmethod)
{
  GetMyClass(data)->floatCMethodMess((float)cmethod);
}

void pix_opencv_contours :: floatApprox1MessCallback(void *data, t_floatarg acc)
{
  GetMyClass(data)->floatAccMess((float)acc);
}
void pix_opencv_contours :: floatApprox2MessCallback(void *data, t_floatarg acc)
{
  GetMyClass(data)->floatApprox2Mess((float)acc);
}
