/*-----------------------------------------------------------------
	LOG
	GEM - Graphics Environment for Multimedia

	Contours detection

	Copyright (c) 2009 IOhannes m zmoelnig. forum::fr::umlute. IEM. zmoelnig@iem.kug.ac.at
	For information on usage and redistribution, and for a DISCLAIMER OF ALL
	WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.

	-----------------------------------------------------------------*/

#ifndef INCLUDE_PIX_OPENCV_CONTOURS_H_
#define INCLUDE_PIX_OPENCV_CONTOURS_H_

#ifndef _EiC
#ifdef __APPLE__
#include <OpenCV/OpenCV.h>
#else
#include "cv.h"
#endif
#endif

#include "Base/GemPixObj.h"

#define MAX_MARKERS 100

/*-----------------------------------------------------------------
	-------------------------------------------------------------------
	CLASS
	pix_opencv_contours
    
	Contours Bounding Rectangle detection

	KEYWORDS
	pix
    
	DESCRIPTION
   
	-----------------------------------------------------------------*/
class GEM_EXTERN pix_opencv_contours : public GemPixObj
{
	CPPEXTERN_HEADER(pix_opencv_contours, GemPixObj)

    public:

	//////////
	// Constructor
	pix_opencv_contours();
    	
 protected:
    	
	//////////
	// Destructor
	virtual ~pix_opencv_contours();

	//////////
	// Do the processing
	virtual void 	processCVImage();


	virtual void 	processRGBAImage(imageStruct &image);
	virtual void 	processRGBImage(imageStruct &image);
	virtual void 	processGrayImage(imageStruct &image); 


	// to detect changes in the image size
	int	comp_xsize;
	int	comp_ysize;

    	
	//////////
	// parameters for the detection
	void	    	floatCModeMess(float);
	void	    	floatCMethodMess(float);

	// contours retrieval mode
	int             x_cmode;
	// contours retrieval method
	int             x_cmethod;

	// contour approximation
	void	    	floatAccMess(float);
	void	    	floatApprox2Mess(float);
	t_float m_approxAccuracy, m_approx2;

 private:
    
	t_outlet 	*m_dataout;
	t_outlet 	*m_countout;
	//////////
	// Static member functions
	static void 	floatCModeMessCallback(void *data, t_floatarg cmode);
	static void 	floatCMethodMessCallback(void *data, t_floatarg cmethod);
	static void 	floatApprox1MessCallback(void *, t_floatarg);
	static void 	floatApprox2MessCallback(void *, t_floatarg);

	////////
	// IplImage needed
	IplImage 	*orig, *gray;
	};

#endif	// for header file
